PrefetchOperation

Struct PrefetchOperation 

pub struct PrefetchOperation<'c> { /* private fields */ }
A prefetch operation. Prefetch operation.

The “prefetch” op prefetches data from a memref location described with subscript indices similar to memref.load, and with three attributes: a read/write specifier, a locality hint, and a cache type specifier as shown below:

memref.prefetch %0[%i, %j], read, locality<3>, data : memref<400x400xi32>

The read/write specifier is either ‘read’ or ‘write’, the locality hint ranges from locality<0> (no locality) to locality<3> (extremely local keep in cache). The cache type specifier is either ‘data’ or ‘instr’ and specifies whether the prefetch is performed on data cache or on instruction cache.

Implementations

Source

impl<'c> PrefetchOperation<'c>

pub fn name() -> &'static str

Returns a name.

pub fn as_operation(&self) -> &Operation<'c>

Returns a generic operation.

pub fn builder( context: &'c Context, location: Location<'c>, ) -> PrefetchOperationBuilder<'c, Unset, Unset, Unset, Unset, Unset>

Creates a builder.

pub fn memref(&self) -> Result<Value<'c, '_>, Error>

pub fn indices(&self) -> impl Iterator<Item = Value<'c, '_>>

pub fn is_write(&self) -> Result<Attribute<'c>, Error>

pub fn set_is_write(&mut self, value: Attribute<'c>)

pub fn locality_hint(&self) -> Result<IntegerAttribute<'c>, Error>

pub fn set_locality_hint(&mut self, value: IntegerAttribute<'c>)

pub fn is_data_cache(&self) -> Result<Attribute<'c>, Error>

pub fn set_is_data_cache(&mut self, value: Attribute<'c>)